#include "Status.h"
#include "Vram.h"
#include "Print.h"
#include "Chars.h"
#include "Sprite.h"
        #include "Init.h"
        #include "EnemyRow.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus()
{
    PrintS(Vram + VramRowSize * 0 + WindowWidth * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 3 + WindowWidth * VramStep, "HIGH");
    PrintS(Vram + VramRowSize * 4 + WindowWidth * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 7 + WindowWidth * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 8 + VramWidth - 2 * VramStep, CurrentStage + 1);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 1 + VramWidth - 6 * VramStep, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 5 + VramWidth - 6 * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 21 + (WindowWidth + 1) * VramStep;
    if (RemainCount > 1) {
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pVram = Put2C(pVram, Char_Remain);
            pVram = Erase2(pVram);
            pVram += VramWidth - 2;
            pVram = PrintC(pVram, i + '0');
            pVram += 1 - VramWidth;
        }
        else {
            do {
                pVram = Put2C(pVram, Char_Remain);
                --i;
            } while (i > 0);
        }
    }
    if (RemainCount < 3) {
        pVram = Erase2(pVram);
    }
}


void PrintGameOver()
{
    constexpr byte Left = (WindowWidth - 8) / 2;
    PrintS(Vram + VramRowSize * (VramHeight / 2) + Left * VramStep, "GAME OVER");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            //	G
            0x1b, 0x29, 0x20, 0x26, 
            0x27, 0x1e, 0x25, 0x25, 
            0x1f, 0x26, 0x1b, 0x2a, 
            0x1b, 0x1f, 0x20, 0x20, 
            //	U
            0x27, 0x1e, 0x1b, 0x2a, 
            0x27, 0x1e, 0x1b, 0x2a, 
            0x27, 0x1e, 0x1b, 0x2a, 
            0x1b, 0x20, 0x20, 0x1c, 
            //	N
            0x27, 0x26, 0x1b, 0x2a, 
            0x27, 0x2a, 0x26, 0x2a, 
            0x27, 0x1e, 0x28, 0x2a, 
            0x1f, 0x1c, 0x1b, 0x20, 
            //	T
            0x1f, 0x28, 0x22, 0x1c, 
            0x1b, 0x27, 0x1e, 0x1b, 
            0x1b, 0x27, 0x1e, 0x1b, 
            0x1b, 0x1f, 0x1c, 0x1b, 
            //	U
            0x27, 0x1e, 0x1b, 0x2a, 
            0x27, 0x1e, 0x1b, 0x2a, 
            0x27, 0x1e, 0x1b, 0x2a, 
            0x1b, 0x20, 0x20, 0x1c, 
            //	S
            0x23, 0x22, 0x20, 0x26, 
            0x1f, 0x26, 0x25, 0x1d, 
            0x23, 0x1d, 0x1b, 0x2a, 
            0x1b, 0x20, 0x20, 0x1c, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (WindowWidth - LogoLength * 4)  * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + (WindowWidth - 19) * VramStep / 2, "PUSH TRIGGER BUTTON");
    PrintS(Vram + VramRowSize * 23 + (WindowWidth - 12) * VramStep, "INUFUTO 2022");
    // {
    //     byte c;
    //     ptr<byte> v;
    //     v = Vram + 32*16;
    //     c = 0;
    //     repeat (128) {
    //         v = Put(v, c);
    //         ++c;
    //         if ((c & 15) == 0) {
    //             v += 32-16;
    //         }
    //     }
    //     UpdateEnemyChars();
    //     SwitchChars();
    //     while (true) {
    //         ++EnemyRowLeft;
    //         UpdateEnemyChars();
    //         SwitchChars();
    //         WaitTimer(30);
    //     }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
}
